@@ -100,74 +100,120 @@ __all__ = (
 ActionParameterTypeDef = TypedDict(
     "ActionParameterTypeDef",
     {
-        "description": NotRequired[str],
-        "required": NotRequired[bool],
+        "description": str,
+        "required": bool,
     },
+    total=False,
 )
 
 ActionSummaryTypeDef = TypedDict(
     "ActionSummaryTypeDef",
     {
-        "id": NotRequired[str],
-        "description": NotRequired[str],
-        "targets": NotRequired[Dict[str, "ActionTargetTypeDef"]],
-        "tags": NotRequired[Dict[str, str]],
+        "id": str,
+        "description": str,
+        "targets": Dict[str, "ActionTargetTypeDef"],
+        "tags": Dict[str, str],
     },
+    total=False,
 )
 
 ActionTargetTypeDef = TypedDict(
     "ActionTargetTypeDef",
     {
-        "resourceType": NotRequired[str],
+        "resourceType": str,
     },
+    total=False,
 )
 
 ActionTypeDef = TypedDict(
     "ActionTypeDef",
     {
-        "id": NotRequired[str],
-        "description": NotRequired[str],
-        "parameters": NotRequired[Dict[str, "ActionParameterTypeDef"]],
-        "targets": NotRequired[Dict[str, "ActionTargetTypeDef"]],
-        "tags": NotRequired[Dict[str, str]],
+        "id": str,
+        "description": str,
+        "parameters": Dict[str, "ActionParameterTypeDef"],
+        "targets": Dict[str, "ActionTargetTypeDef"],
+        "tags": Dict[str, str],
     },
+    total=False,
 )
 
-CreateExperimentTemplateActionInputTypeDef = TypedDict(
-    "CreateExperimentTemplateActionInputTypeDef",
+_RequiredCreateExperimentTemplateActionInputTypeDef = TypedDict(
+    "_RequiredCreateExperimentTemplateActionInputTypeDef",
     {
         "actionId": str,
-        "description": NotRequired[str],
-        "parameters": NotRequired[Mapping[str, str]],
-        "targets": NotRequired[Mapping[str, str]],
-        "startAfter": NotRequired[Sequence[str]],
     },
 )
+_OptionalCreateExperimentTemplateActionInputTypeDef = TypedDict(
+    "_OptionalCreateExperimentTemplateActionInputTypeDef",
+    {
+        "description": str,
+        "parameters": Mapping[str, str],
+        "targets": Mapping[str, str],
+        "startAfter": Sequence[str],
+    },
+    total=False,
+)
 
-CreateExperimentTemplateLogConfigurationInputTypeDef = TypedDict(
-    "CreateExperimentTemplateLogConfigurationInputTypeDef",
+
+class CreateExperimentTemplateActionInputTypeDef(
+    _RequiredCreateExperimentTemplateActionInputTypeDef,
+    _OptionalCreateExperimentTemplateActionInputTypeDef,
+):
+    pass
+
+
+_RequiredCreateExperimentTemplateLogConfigurationInputTypeDef = TypedDict(
+    "_RequiredCreateExperimentTemplateLogConfigurationInputTypeDef",
     {
         "logSchemaVersion": int,
-        "cloudWatchLogsConfiguration": NotRequired[
-            "ExperimentTemplateCloudWatchLogsLogConfigurationInputTypeDef"
-        ],
-        "s3Configuration": NotRequired["ExperimentTemplateS3LogConfigurationInputTypeDef"],
     },
 )
+_OptionalCreateExperimentTemplateLogConfigurationInputTypeDef = TypedDict(
+    "_OptionalCreateExperimentTemplateLogConfigurationInputTypeDef",
+    {
+        "cloudWatchLogsConfiguration": (
+            "ExperimentTemplateCloudWatchLogsLogConfigurationInputTypeDef"
+        ),
+        "s3Configuration": "ExperimentTemplateS3LogConfigurationInputTypeDef",
+    },
+    total=False,
+)
 
-CreateExperimentTemplateRequestRequestTypeDef = TypedDict(
-    "CreateExperimentTemplateRequestRequestTypeDef",
+
+class CreateExperimentTemplateLogConfigurationInputTypeDef(
+    _RequiredCreateExperimentTemplateLogConfigurationInputTypeDef,
+    _OptionalCreateExperimentTemplateLogConfigurationInputTypeDef,
+):
+    pass
+
+
+_RequiredCreateExperimentTemplateRequestRequestTypeDef = TypedDict(
+    "_RequiredCreateExperimentTemplateRequestRequestTypeDef",
     {
         "clientToken": str,
         "description": str,
         "stopConditions": Sequence["CreateExperimentTemplateStopConditionInputTypeDef"],
         "actions": Mapping[str, "CreateExperimentTemplateActionInputTypeDef"],
         "roleArn": str,
-        "targets": NotRequired[Mapping[str, "CreateExperimentTemplateTargetInputTypeDef"]],
-        "tags": NotRequired[Mapping[str, str]],
-        "logConfiguration": NotRequired["CreateExperimentTemplateLogConfigurationInputTypeDef"],
     },
 )
+_OptionalCreateExperimentTemplateRequestRequestTypeDef = TypedDict(
+    "_OptionalCreateExperimentTemplateRequestRequestTypeDef",
+    {
+        "targets": Mapping[str, "CreateExperimentTemplateTargetInputTypeDef"],
+        "tags": Mapping[str, str],
+        "logConfiguration": "CreateExperimentTemplateLogConfigurationInputTypeDef",
+    },
+    total=False,
+)
+
+
+class CreateExperimentTemplateRequestRequestTypeDef(
+    _RequiredCreateExperimentTemplateRequestRequestTypeDef,
+    _OptionalCreateExperimentTemplateRequestRequestTypeDef,
+):
+    pass
+
 
 CreateExperimentTemplateResponseTypeDef = TypedDict(
     "CreateExperimentTemplateResponseTypeDef",
@@ -177,25 +223,53 @@ CreateExperimentTemplateResponseTypeDef = TypedDict(
     },
 )
 
-CreateExperimentTemplateStopConditionInputTypeDef = TypedDict(
-    "CreateExperimentTemplateStopConditionInputTypeDef",
+_RequiredCreateExperimentTemplateStopConditionInputTypeDef = TypedDict(
+    "_RequiredCreateExperimentTemplateStopConditionInputTypeDef",
     {
         "source": str,
-        "value": NotRequired[str],
     },
 )
+_OptionalCreateExperimentTemplateStopConditionInputTypeDef = TypedDict(
+    "_OptionalCreateExperimentTemplateStopConditionInputTypeDef",
+    {
+        "value": str,
+    },
+    total=False,
+)
 
-CreateExperimentTemplateTargetInputTypeDef = TypedDict(
-    "CreateExperimentTemplateTargetInputTypeDef",
+
+class CreateExperimentTemplateStopConditionInputTypeDef(
+    _RequiredCreateExperimentTemplateStopConditionInputTypeDef,
+    _OptionalCreateExperimentTemplateStopConditionInputTypeDef,
+):
+    pass
+
+
+_RequiredCreateExperimentTemplateTargetInputTypeDef = TypedDict(
+    "_RequiredCreateExperimentTemplateTargetInputTypeDef",
     {
         "resourceType": str,
         "selectionMode": str,
-        "resourceArns": NotRequired[Sequence[str]],
-        "resourceTags": NotRequired[Mapping[str, str]],
-        "filters": NotRequired[Sequence["ExperimentTemplateTargetInputFilterTypeDef"]],
-        "parameters": NotRequired[Mapping[str, str]],
     },
 )
+_OptionalCreateExperimentTemplateTargetInputTypeDef = TypedDict(
+    "_OptionalCreateExperimentTemplateTargetInputTypeDef",
+    {
+        "resourceArns": Sequence[str],
+        "resourceTags": Mapping[str, str],
+        "filters": Sequence["ExperimentTemplateTargetInputFilterTypeDef"],
+        "parameters": Mapping[str, str],
+    },
+    total=False,
+)
+
+
+class CreateExperimentTemplateTargetInputTypeDef(
+    _RequiredCreateExperimentTemplateTargetInputTypeDef,
+    _OptionalCreateExperimentTemplateTargetInputTypeDef,
+):
+    pass
+
 
 DeleteExperimentTemplateRequestRequestTypeDef = TypedDict(
     "DeleteExperimentTemplateRequestRequestTypeDef",
@@ -215,107 +289,116 @@ DeleteExperimentTemplateResponseTypeDef = TypedDict(
 ExperimentActionStateTypeDef = TypedDict(
     "ExperimentActionStateTypeDef",
     {
-        "status": NotRequired[ExperimentActionStatusType],
-        "reason": NotRequired[str],
+        "status": ExperimentActionStatusType,
+        "reason": str,
     },
+    total=False,
 )
 
 ExperimentActionTypeDef = TypedDict(
     "ExperimentActionTypeDef",
     {
-        "actionId": NotRequired[str],
-        "description": NotRequired[str],
-        "parameters": NotRequired[Dict[str, str]],
-        "targets": NotRequired[Dict[str, str]],
-        "startAfter": NotRequired[List[str]],
-        "state": NotRequired["ExperimentActionStateTypeDef"],
-        "startTime": NotRequired[datetime],
-        "endTime": NotRequired[datetime],
+        "actionId": str,
+        "description": str,
+        "parameters": Dict[str, str],
+        "targets": Dict[str, str],
+        "startAfter": List[str],
+        "state": "ExperimentActionStateTypeDef",
+        "startTime": datetime,
+        "endTime": datetime,
     },
+    total=False,
 )
 
 ExperimentCloudWatchLogsLogConfigurationTypeDef = TypedDict(
     "ExperimentCloudWatchLogsLogConfigurationTypeDef",
     {
-        "logGroupArn": NotRequired[str],
+        "logGroupArn": str,
     },
+    total=False,
 )
 
 ExperimentLogConfigurationTypeDef = TypedDict(
     "ExperimentLogConfigurationTypeDef",
     {
-        "cloudWatchLogsConfiguration": NotRequired[
-            "ExperimentCloudWatchLogsLogConfigurationTypeDef"
-        ],
-        "s3Configuration": NotRequired["ExperimentS3LogConfigurationTypeDef"],
-        "logSchemaVersion": NotRequired[int],
+        "cloudWatchLogsConfiguration": "ExperimentCloudWatchLogsLogConfigurationTypeDef",
+        "s3Configuration": "ExperimentS3LogConfigurationTypeDef",
+        "logSchemaVersion": int,
     },
+    total=False,
 )
 
 ExperimentS3LogConfigurationTypeDef = TypedDict(
     "ExperimentS3LogConfigurationTypeDef",
     {
-        "bucketName": NotRequired[str],
-        "prefix": NotRequired[str],
+        "bucketName": str,
+        "prefix": str,
     },
+    total=False,
 )
 
 ExperimentStateTypeDef = TypedDict(
     "ExperimentStateTypeDef",
     {
-        "status": NotRequired[ExperimentStatusType],
-        "reason": NotRequired[str],
+        "status": ExperimentStatusType,
+        "reason": str,
     },
+    total=False,
 )
 
 ExperimentStopConditionTypeDef = TypedDict(
     "ExperimentStopConditionTypeDef",
     {
-        "source": NotRequired[str],
-        "value": NotRequired[str],
+        "source": str,
+        "value": str,
     },
+    total=False,
 )
 
 ExperimentSummaryTypeDef = TypedDict(
     "ExperimentSummaryTypeDef",
     {
-        "id": NotRequired[str],
-        "experimentTemplateId": NotRequired[str],
-        "state": NotRequired["ExperimentStateTypeDef"],
-        "creationTime": NotRequired[datetime],
-        "tags": NotRequired[Dict[str, str]],
+        "id": str,
+        "experimentTemplateId": str,
+        "state": "ExperimentStateTypeDef",
+        "creationTime": datetime,
+        "tags": Dict[str, str],
     },
+    total=False,
 )
 
 ExperimentTargetFilterTypeDef = TypedDict(
     "ExperimentTargetFilterTypeDef",
     {
-        "path": NotRequired[str],
-        "values": NotRequired[List[str]],
+        "path": str,
+        "values": List[str],
     },
+    total=False,
 )
 
 ExperimentTargetTypeDef = TypedDict(
     "ExperimentTargetTypeDef",
     {
-        "resourceType": NotRequired[str],
-        "resourceArns": NotRequired[List[str]],
-        "resourceTags": NotRequired[Dict[str, str]],
-        "filters": NotRequired[List["ExperimentTargetFilterTypeDef"]],
-        "selectionMode": NotRequired[str],
-        "parameters": NotRequired[Dict[str, str]],
+        "resourceType": str,
+        "resourceArns": List[str],
+        "resourceTags": Dict[str, str],
+        "filters": List["ExperimentTargetFilterTypeDef"],
+        "selectionMode": str,
+        "parameters": Dict[str, str],
     },
+    total=False,
 )
 
 ExperimentTemplateActionTypeDef = TypedDict(
     "ExperimentTemplateActionTypeDef",
     {
-        "actionId": NotRequired[str],
-        "description": NotRequired[str],
-        "parameters": NotRequired[Dict[str, str]],
-        "targets": NotRequired[Dict[str, str]],
-        "startAfter": NotRequired[List[str]],
+        "actionId": str,
+        "description": str,
+        "parameters": Dict[str, str],
+        "targets": Dict[str, str],
+        "startAfter": List[str],
     },
+    total=False,
 )
 
 ExperimentTemplateCloudWatchLogsLogConfigurationInputTypeDef = TypedDict(
@@ -328,62 +411,80 @@ ExperimentTemplateCloudWatchLogsLogConfigurationInputTypeDef = TypedDict(
 ExperimentTemplateCloudWatchLogsLogConfigurationTypeDef = TypedDict(
     "ExperimentTemplateCloudWatchLogsLogConfigurationTypeDef",
     {
-        "logGroupArn": NotRequired[str],
+        "logGroupArn": str,
     },
+    total=False,
 )
 
 ExperimentTemplateLogConfigurationTypeDef = TypedDict(
     "ExperimentTemplateLogConfigurationTypeDef",
     {
-        "cloudWatchLogsConfiguration": NotRequired[
-            "ExperimentTemplateCloudWatchLogsLogConfigurationTypeDef"
-        ],
-        "s3Configuration": NotRequired["ExperimentTemplateS3LogConfigurationTypeDef"],
-        "logSchemaVersion": NotRequired[int],
+        "cloudWatchLogsConfiguration": "ExperimentTemplateCloudWatchLogsLogConfigurationTypeDef",
+        "s3Configuration": "ExperimentTemplateS3LogConfigurationTypeDef",
+        "logSchemaVersion": int,
     },
+    total=False,
 )
 
-ExperimentTemplateS3LogConfigurationInputTypeDef = TypedDict(
-    "ExperimentTemplateS3LogConfigurationInputTypeDef",
+_RequiredExperimentTemplateS3LogConfigurationInputTypeDef = TypedDict(
+    "_RequiredExperimentTemplateS3LogConfigurationInputTypeDef",
     {
         "bucketName": str,
-        "prefix": NotRequired[str],
     },
 )
+_OptionalExperimentTemplateS3LogConfigurationInputTypeDef = TypedDict(
+    "_OptionalExperimentTemplateS3LogConfigurationInputTypeDef",
+    {
+        "prefix": str,
+    },
+    total=False,
+)
+
+
+class ExperimentTemplateS3LogConfigurationInputTypeDef(
+    _RequiredExperimentTemplateS3LogConfigurationInputTypeDef,
+    _OptionalExperimentTemplateS3LogConfigurationInputTypeDef,
+):
+    pass
+
 
 ExperimentTemplateS3LogConfigurationTypeDef = TypedDict(
     "ExperimentTemplateS3LogConfigurationTypeDef",
     {
-        "bucketName": NotRequired[str],
-        "prefix": NotRequired[str],
+        "bucketName": str,
+        "prefix": str,
     },
+    total=False,
 )
 
 ExperimentTemplateStopConditionTypeDef = TypedDict(
     "ExperimentTemplateStopConditionTypeDef",
     {
-        "source": NotRequired[str],
-        "value": NotRequired[str],
+        "source": str,
+        "value": str,
     },
+    total=False,
 )
 
 ExperimentTemplateSummaryTypeDef = TypedDict(
     "ExperimentTemplateSummaryTypeDef",
     {
-        "id": NotRequired[str],
-        "description": NotRequired[str],
-        "creationTime": NotRequired[datetime],
-        "lastUpdateTime": NotRequired[datetime],
-        "tags": NotRequired[Dict[str, str]],
+        "id": str,
+        "description": str,
+        "creationTime": datetime,
+        "lastUpdateTime": datetime,
+        "tags": Dict[str, str],
     },
+    total=False,
 )
 
 ExperimentTemplateTargetFilterTypeDef = TypedDict(
     "ExperimentTemplateTargetFilterTypeDef",
     {
-        "path": NotRequired[str],
-        "values": NotRequired[List[str]],
+        "path": str,
+        "values": List[str],
     },
+    total=False,
 )
 
 ExperimentTemplateTargetInputFilterTypeDef = TypedDict(
@@ -397,47 +498,50 @@ ExperimentTemplateTargetInputFilterTypeDef = TypedDict(
 ExperimentTemplateTargetTypeDef = TypedDict(
     "ExperimentTemplateTargetTypeDef",
     {
-        "resourceType": NotRequired[str],
-        "resourceArns": NotRequired[List[str]],
-        "resourceTags": NotRequired[Dict[str, str]],
-        "filters": NotRequired[List["ExperimentTemplateTargetFilterTypeDef"]],
-        "selectionMode": NotRequired[str],
-        "parameters": NotRequired[Dict[str, str]],
+        "resourceType": str,
+        "resourceArns": List[str],
+        "resourceTags": Dict[str, str],
+        "filters": List["ExperimentTemplateTargetFilterTypeDef"],
+        "selectionMode": str,
+        "parameters": Dict[str, str],
     },
+    total=False,
 )
 
 ExperimentTemplateTypeDef = TypedDict(
     "ExperimentTemplateTypeDef",
     {
-        "id": NotRequired[str],
-        "description": NotRequired[str],
-        "targets": NotRequired[Dict[str, "ExperimentTemplateTargetTypeDef"]],
-        "actions": NotRequired[Dict[str, "ExperimentTemplateActionTypeDef"]],
-        "stopConditions": NotRequired[List["ExperimentTemplateStopConditionTypeDef"]],
-        "creationTime": NotRequired[datetime],
-        "lastUpdateTime": NotRequired[datetime],
-        "roleArn": NotRequired[str],
-        "tags": NotRequired[Dict[str, str]],
-        "logConfiguration": NotRequired["ExperimentTemplateLogConfigurationTypeDef"],
+        "id": str,
+        "description": str,
+        "targets": Dict[str, "ExperimentTemplateTargetTypeDef"],
+        "actions": Dict[str, "ExperimentTemplateActionTypeDef"],
+        "stopConditions": List["ExperimentTemplateStopConditionTypeDef"],
+        "creationTime": datetime,
+        "lastUpdateTime": datetime,
+        "roleArn": str,
+        "tags": Dict[str, str],
+        "logConfiguration": "ExperimentTemplateLogConfigurationTypeDef",
     },
+    total=False,
 )
 
 ExperimentTypeDef = TypedDict(
     "ExperimentTypeDef",
     {
-        "id": NotRequired[str],
-        "experimentTemplateId": NotRequired[str],
-        "roleArn": NotRequired[str],
-        "state": NotRequired["ExperimentStateTypeDef"],
-        "targets": NotRequired[Dict[str, "ExperimentTargetTypeDef"]],
-        "actions": NotRequired[Dict[str, "ExperimentActionTypeDef"]],
-        "stopConditions": NotRequired[List["ExperimentStopConditionTypeDef"]],
-        "creationTime": NotRequired[datetime],
-        "startTime": NotRequired[datetime],
-        "endTime": NotRequired[datetime],
-        "tags": NotRequired[Dict[str, str]],
-        "logConfiguration": NotRequired["ExperimentLogConfigurationTypeDef"],
+        "id": str,
+        "experimentTemplateId": str,
+        "roleArn": str,
+        "state": "ExperimentStateTypeDef",
+        "targets": Dict[str, "ExperimentTargetTypeDef"],
+        "actions": Dict[str, "ExperimentActionTypeDef"],
+        "stopConditions": List["ExperimentStopConditionTypeDef"],
+        "creationTime": datetime,
+        "startTime": datetime,
+        "endTime": datetime,
+        "tags": Dict[str, str],
+        "logConfiguration": "ExperimentLogConfigurationTypeDef",
     },
+    total=False,
 )
 
 GetActionRequestRequestTypeDef = TypedDict(
@@ -503,9 +607,10 @@ GetTargetResourceTypeResponseTypeDef = TypedDict(
 ListActionsRequestRequestTypeDef = TypedDict(
     "ListActionsRequestRequestTypeDef",
     {
-        "maxResults": NotRequired[int],
-        "nextToken": NotRequired[str],
+        "maxResults": int,
+        "nextToken": str,
     },
+    total=False,
 )
 
 ListActionsResponseTypeDef = TypedDict(
@@ -520,9 +625,10 @@ ListActionsResponseTypeDef = TypedDict(
 ListExperimentTemplatesRequestRequestTypeDef = TypedDict(
     "ListExperimentTemplatesRequestRequestTypeDef",
     {
-        "maxResults": NotRequired[int],
-        "nextToken": NotRequired[str],
+        "maxResults": int,
+        "nextToken": str,
     },
+    total=False,
 )
 
 ListExperimentTemplatesResponseTypeDef = TypedDict(
@@ -537,9 +643,10 @@ ListExperimentTemplatesResponseTypeDef = TypedDict(
 ListExperimentsRequestRequestTypeDef = TypedDict(
     "ListExperimentsRequestRequestTypeDef",
     {
-        "maxResults": NotRequired[int],
-        "nextToken": NotRequired[str],
+        "maxResults": int,
+        "nextToken": str,
     },
+    total=False,
 )
 
 ListExperimentsResponseTypeDef = TypedDict(
@@ -569,9 +676,10 @@ ListTagsForResourceResponseTypeDef = TypedDict(
 ListTargetResourceTypesRequestRequestTypeDef = TypedDict(
     "ListTargetResourceTypesRequestRequestTypeDef",
     {
-        "maxResults": NotRequired[int],
-        "nextToken": NotRequired[str],
+        "maxResults": int,
+        "nextToken": str,
     },
+    total=False,
 )
 
 ListTargetResourceTypesResponseTypeDef = TypedDict(
@@ -594,14 +702,27 @@ ResponseMetadataTypeDef = TypedDict(
     },
 )
 
-StartExperimentRequestRequestTypeDef = TypedDict(
-    "StartExperimentRequestRequestTypeDef",
+_RequiredStartExperimentRequestRequestTypeDef = TypedDict(
+    "_RequiredStartExperimentRequestRequestTypeDef",
     {
         "clientToken": str,
         "experimentTemplateId": str,
-        "tags": NotRequired[Mapping[str, str]],
     },
 )
+_OptionalStartExperimentRequestRequestTypeDef = TypedDict(
+    "_OptionalStartExperimentRequestRequestTypeDef",
+    {
+        "tags": Mapping[str, str],
+    },
+    total=False,
+)
+
+
+class StartExperimentRequestRequestTypeDef(
+    _RequiredStartExperimentRequestRequestTypeDef, _OptionalStartExperimentRequestRequestTypeDef
+):
+    pass
+
 
 StartExperimentResponseTypeDef = TypedDict(
     "StartExperimentResponseTypeDef",
@@ -637,72 +758,102 @@ TagResourceRequestRequestTypeDef = TypedDict(
 TargetResourceTypeParameterTypeDef = TypedDict(
     "TargetResourceTypeParameterTypeDef",
     {
-        "description": NotRequired[str],
-        "required": NotRequired[bool],
+        "description": str,
+        "required": bool,
     },
+    total=False,
 )
 
 TargetResourceTypeSummaryTypeDef = TypedDict(
     "TargetResourceTypeSummaryTypeDef",
     {
-        "resourceType": NotRequired[str],
-        "description": NotRequired[str],
+        "resourceType": str,
+        "description": str,
     },
+    total=False,
 )
 
 TargetResourceTypeTypeDef = TypedDict(
     "TargetResourceTypeTypeDef",
     {
-        "resourceType": NotRequired[str],
-        "description": NotRequired[str],
-        "parameters": NotRequired[Dict[str, "TargetResourceTypeParameterTypeDef"]],
+        "resourceType": str,
+        "description": str,
+        "parameters": Dict[str, "TargetResourceTypeParameterTypeDef"],
     },
+    total=False,
 )
 
-UntagResourceRequestRequestTypeDef = TypedDict(
-    "UntagResourceRequestRequestTypeDef",
+_RequiredUntagResourceRequestRequestTypeDef = TypedDict(
+    "_RequiredUntagResourceRequestRequestTypeDef",
     {
         "resourceArn": str,
-        "tagKeys": NotRequired[Sequence[str]],
     },
 )
+_OptionalUntagResourceRequestRequestTypeDef = TypedDict(
+    "_OptionalUntagResourceRequestRequestTypeDef",
+    {
+        "tagKeys": Sequence[str],
+    },
+    total=False,
+)
+
+
+class UntagResourceRequestRequestTypeDef(
+    _RequiredUntagResourceRequestRequestTypeDef, _OptionalUntagResourceRequestRequestTypeDef
+):
+    pass
+
 
 UpdateExperimentTemplateActionInputItemTypeDef = TypedDict(
     "UpdateExperimentTemplateActionInputItemTypeDef",
     {
-        "actionId": NotRequired[str],
-        "description": NotRequired[str],
-        "parameters": NotRequired[Mapping[str, str]],
-        "targets": NotRequired[Mapping[str, str]],
-        "startAfter": NotRequired[Sequence[str]],
+        "actionId": str,
+        "description": str,
+        "parameters": Mapping[str, str],
+        "targets": Mapping[str, str],
+        "startAfter": Sequence[str],
     },
+    total=False,
 )
 
 UpdateExperimentTemplateLogConfigurationInputTypeDef = TypedDict(
     "UpdateExperimentTemplateLogConfigurationInputTypeDef",
     {
-        "cloudWatchLogsConfiguration": NotRequired[
+        "cloudWatchLogsConfiguration": (
             "ExperimentTemplateCloudWatchLogsLogConfigurationInputTypeDef"
-        ],
-        "s3Configuration": NotRequired["ExperimentTemplateS3LogConfigurationInputTypeDef"],
-        "logSchemaVersion": NotRequired[int],
+        ),
+        "s3Configuration": "ExperimentTemplateS3LogConfigurationInputTypeDef",
+        "logSchemaVersion": int,
     },
+    total=False,
 )
 
-UpdateExperimentTemplateRequestRequestTypeDef = TypedDict(
-    "UpdateExperimentTemplateRequestRequestTypeDef",
+_RequiredUpdateExperimentTemplateRequestRequestTypeDef = TypedDict(
+    "_RequiredUpdateExperimentTemplateRequestRequestTypeDef",
     {
         "id": str,
-        "description": NotRequired[str],
-        "stopConditions": NotRequired[
-            Sequence["UpdateExperimentTemplateStopConditionInputTypeDef"]
-        ],
-        "targets": NotRequired[Mapping[str, "UpdateExperimentTemplateTargetInputTypeDef"]],
-        "actions": NotRequired[Mapping[str, "UpdateExperimentTemplateActionInputItemTypeDef"]],
-        "roleArn": NotRequired[str],
-        "logConfiguration": NotRequired["UpdateExperimentTemplateLogConfigurationInputTypeDef"],
     },
 )
+_OptionalUpdateExperimentTemplateRequestRequestTypeDef = TypedDict(
+    "_OptionalUpdateExperimentTemplateRequestRequestTypeDef",
+    {
+        "description": str,
+        "stopConditions": Sequence["UpdateExperimentTemplateStopConditionInputTypeDef"],
+        "targets": Mapping[str, "UpdateExperimentTemplateTargetInputTypeDef"],
+        "actions": Mapping[str, "UpdateExperimentTemplateActionInputItemTypeDef"],
+        "roleArn": str,
+        "logConfiguration": "UpdateExperimentTemplateLogConfigurationInputTypeDef",
+    },
+    total=False,
+)
+
+
+class UpdateExperimentTemplateRequestRequestTypeDef(
+    _RequiredUpdateExperimentTemplateRequestRequestTypeDef,
+    _OptionalUpdateExperimentTemplateRequestRequestTypeDef,
+):
+    pass
+
 
 UpdateExperimentTemplateResponseTypeDef = TypedDict(
     "UpdateExperimentTemplateResponseTypeDef",
@@ -712,22 +863,49 @@ UpdateExperimentTemplateResponseTypeDef = TypedDict(
     },
 )
 
-UpdateExperimentTemplateStopConditionInputTypeDef = TypedDict(
-    "UpdateExperimentTemplateStopConditionInputTypeDef",
+_RequiredUpdateExperimentTemplateStopConditionInputTypeDef = TypedDict(
+    "_RequiredUpdateExperimentTemplateStopConditionInputTypeDef",
     {
         "source": str,
-        "value": NotRequired[str],
     },
 )
+_OptionalUpdateExperimentTemplateStopConditionInputTypeDef = TypedDict(
+    "_OptionalUpdateExperimentTemplateStopConditionInputTypeDef",
+    {
+        "value": str,
+    },
+    total=False,
+)
 
-UpdateExperimentTemplateTargetInputTypeDef = TypedDict(
-    "UpdateExperimentTemplateTargetInputTypeDef",
+
+class UpdateExperimentTemplateStopConditionInputTypeDef(
+    _RequiredUpdateExperimentTemplateStopConditionInputTypeDef,
+    _OptionalUpdateExperimentTemplateStopConditionInputTypeDef,
+):
+    pass
+
+
+_RequiredUpdateExperimentTemplateTargetInputTypeDef = TypedDict(
+    "_RequiredUpdateExperimentTemplateTargetInputTypeDef",
     {
         "resourceType": str,
         "selectionMode": str,
-        "resourceArns": NotRequired[Sequence[str]],
-        "resourceTags": NotRequired[Mapping[str, str]],
-        "filters": NotRequired[Sequence["ExperimentTemplateTargetInputFilterTypeDef"]],
-        "parameters": NotRequired[Mapping[str, str]],
     },
 )
+_OptionalUpdateExperimentTemplateTargetInputTypeDef = TypedDict(
+    "_OptionalUpdateExperimentTemplateTargetInputTypeDef",
+    {
+        "resourceArns": Sequence[str],
+        "resourceTags": Mapping[str, str],
+        "filters": Sequence["ExperimentTemplateTargetInputFilterTypeDef"],
+        "parameters": Mapping[str, str],
+    },
+    total=False,
+)
+
+
+class UpdateExperimentTemplateTargetInputTypeDef(
+    _RequiredUpdateExperimentTemplateTargetInputTypeDef,
+    _OptionalUpdateExperimentTemplateTargetInputTypeDef,
+):
+    pass
